#!/usr/bin/perl -w

# Copyright 2013, 2014, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz tree which is the Calkin-Wilf tree starting filtered to p,q
# not both odd, by a box drawn around the desired both-odd points.

use strict;
use Math::PlanePath::RationalsTree;

use FindBin;
use lib $FindBin::Bin;
use MyTriples 'pq_acceptable';


print "  % generated by tools/$FindBin::Script\n";

my $tree_type = 'CW';
my $depth_limit = 4;
# $tree_type = 'HCS';
# $depth_limit = 5;
my $path = Math::PlanePath::RationalsTree->new(tree_type => $tree_type);
my $n_top = $path->xy_to_n(1,1);
my $depth_top = $path->tree_n_to_depth($n_top);

my $descend;
$descend = sub {
  my ($n) = @_;
  my $depth = $path->tree_n_to_depth($n) - $depth_top;
  my ($x,$y) = $path->n_to_xy($n);
  my $indent = '  'x($depth+1);

  my $draw = (pq_acceptable($x,$y) || pq_acceptable($y,$x)
              ? "[draw]" : "");
  my $str = ($depth <= $depth_limit ? "$x,$y" : "");
  print "${indent}",
    ($n == $n_top ? '\\' : ''),
    "node$draw {$str}\n";
  return if $depth >= $depth_limit;

  my @n_children = $path->tree_n_children($n);
  foreach my $i (0 .. $#n_children) {
    my $n_child = $n_children[$i];
    print "${indent}child {\n";
    $descend->($n_child);
    print "${indent}}",
      ($n==$n_top && $i==$#n_children ? ';' : ''),  # final semi
      "\n";
  }
};
$descend->($n_top);

print "  % end generated\n";
1;
