#!/usr/bin/perl -w

# Copyright 2013, 2014, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz picture of UAD low-to-high tree branches.

use 5.010;
use strict;
use FindBin;
use Math::Libm 'hypot';
use Math::PlanePath::PythagoreanTree;


my $max_depth = 2;

print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                 digit_order => 'LtoH');
foreach my $n ($path->n_start .. $path->tree_depth_to_n_end($max_depth)) {
  my $depth = $path->tree_n_to_depth($n);
  if ($n == $path->tree_depth_to_n($depth)) {
    print "  %\n";
    print "  % nodes at depth=$depth, lines to parent\n";
  }

  my ($x,$y) = $path->n_to_xy($n);
  my $n_parent = $path->tree_n_parent($n);
  if (defined $n_parent) {
    my ($x_parent,$y_parent) = $path->n_to_xy($n_parent);
    print "  \\filldraw ($x,$y) circle -- ($x_parent,$y_parent);\n";
  } else {
    print "  \\filldraw ($x,$y) circle;\n";
  }
}

my $end_length = .4;
print "  %\n";
print "  % nodes at depth=$max_depth thin lines to children\n";
print "  \\begin{scope}[ultra thin]\n";
foreach my $n ($path->tree_depth_to_n($max_depth)
               .. $path->tree_depth_to_n_end($max_depth)) {
  my ($x,$y) = $path->n_to_xy($n);
  foreach my $n_child ($path->tree_n_children($n)) {
    my ($x_child,$y_child) = $path->n_to_xy($n_child);

    my $dx = $x_child - $x;
    my $dy = $y_child - $y;
    my $h = hypot($dx,$dy);
    $dx *= $end_length/$h;
    $dy *= $end_length/$h;
    $x_child = $x + $dx;
    $y_child = $y + $dy;
    $x_child = sprintf '%.3f', $x_child;
    $y_child = sprintf '%.3f', $y_child;
    print "  \\draw ($x,$y) circle -- ($x_child,$y_child);\n";
  }
}
print "  \\end{scope}\n";
print "  % end generated\n";
1;
