#!/usr/bin/perl -w

# Copyright 2013, 2014, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz picture which is UArD depth=2 sample row showing horizontal
# and diagonal lines.

use strict;
use FindBin;
use Math::PlanePath::PythagoreanTree;

my $depth = 2;

print "    % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                 tree_type => 'UArD');
foreach my $n ($path->tree_depth_to_n($depth) .. $path->tree_depth_to_n_end($depth)) {
  my $n_row = $path->tree_depth_to_n($depth);
  my $offset = $n - $n_row;

  my ($x,$y) = $path->n_to_xy($n);
  print "    \\node (p$x$y) at ($x,$y) [my box] {$x,$y};\n";
  if ($offset > 0) {
    my ($x1,$y1) = $path->n_to_xy($n-1);

    if ($x1==9 && $y1==4 && $x==6 && $y==1) {
      # tweak attach points for 9,4->6,1 which overlaps 8,3->7,2
      print "    % \\draw[->] (p$x1$y1.\\MyAngle) -- (p$x$y.{-90-\\MyAngle});\n";
    } else {
      print "    \\draw[->] (p$x1$y1) -- (p$x$y);\n";
    }
  }
}

print "    % end generated\n";
exit 0;

# print "\\draw[->]\n";
# my $join = '';
# foreach my $n ($path->tree_depth_to_n($depth) .. $path->tree_depth_to_n_end($depth)) {
#   my $n_row = $path->tree_depth_to_n($depth);
#   my $offset = $n - $n_row;
#
#   my ($x,$y) = $path->n_to_xy($n);
#   printf "  %s(%d,%d) node{%d,%d}\n", $join, $x,$y, $x,$y;
#   $join = ' -- ';
# }
# print ";\n";
